function CalendarController($scope, $rootScope, templateManager, ScreenService, ZhUtil, Query, EventChildPropagation, metaDataFactory) {

	var dateFormat = $scope.field.dateFormat || moment().locale(window.lang).localeData().longDateFormat("L");
	var displayFormat = $scope.field.displayFormat || dateFormat;
	var DEFAULT_DATE_FORMAT = "YYYY-MM-DD";

	$scope.openField = function(field) {

		if (field.isReadOnly()) {
			return;
		}
		var widgetEvents = [{
			"name": "WidgetAfterMoveRow",
			"code": "args.owner.setSelected(args);",
			"id": 9999
		}];
		var calendarWidget = {
			"id": 0,
			"name": "Widget",
			"label": ScreenService.i18n(field.label),
			"template": "widget/calendar.html",
			"container": field.widget.container,
			"selectField": field,
			"isVisible": true,
			"popupNoBlock": false,
			"showDescriptionOrder": false,
			"dataSource": field.dataSource,
			"dataSourceFilter": field.dataSourceFilter,
			"todayText": field.todayText || ScreenService.i18n("Today"),
			"weekDays": field.weekDays,
			"monthNames": field.monthNames || moment.months(),
			"dateField": field.dateField,
			"titleField": field.titleField,
			"dateFormat": dateFormat,
			"fields": [],
			"field": field,
			"calendarMode": field.calendarMode,
			"actions": [{
				"label": ScreenService.i18n("Cancel"),
				"hideIcon": true,
				"showAsAction": "back",
				"isVisible": true,
				"showOnForm": true,
				"showOnList": true,
				"events": [{
					"name": "ActionEvent",
					"code": "ScreenService.closePopup()",
					"id": 9998
				}]
			}],
			"events": EventChildPropagation.propagate(widgetEvents, field)
		};
		field.calendarWidget = field.calendarWidget || metaDataFactory.widgetFactory(calendarWidget);

		if (field.maxDate) {
			field.calendarWidget.maxDate = field.maxDate;
		}

		if (field.minDate) {
			field.calendarWidget.minDate = field.minDate;
		}

		field.calendarWidget.setSelected = function(args) {
			ScreenService.closePopup();

			var field = args.owner.selectField;
			field.widget.currentRow[field.name] = args.row;
			field.useDateFormat = true;

			if (field.change) {
				field.change();
			}

		};

		field.clearValue = $scope.clearField;

		ScreenService.openPopup(field.calendarWidget);
	};

	var oldFieldValue = $scope.field.value;
	$scope.field.value = function(value) {
		if(!value){
			return $scope.field.widget.currentRow[$scope.field.name];
		}
		var dateFormat = $scope.field.dateFormat || "L";
		value = moment(value, dateFormat).format(dateFormat);
		$scope.field.widget.currentRow[$scope.field.name] = value;
	};

	$scope.field.openField = function() {
		$scope.openField($scope.field);
	};

	if ($scope.startField) {
		$scope.startField.openField = function() {
			$scope.openField($scope.startField);
		};
	}

	if ($scope.endField) {
		$scope.endField.openField = function() {
			$scope.openField($scope.endField);
		};
	}

	$scope.clearField = function(field) {
		if (!field.isReadOnly()) {
			field.widget.currentRow[field.name] = "";
			field.currentDate = moment().startOf('month').startOf('day');
			if (field.change) {
				field.change();
			}
		}
	};

	$scope.dynamicLongDateFormat = function() {
		return displayFormat.replace(/[D-Y]/g, '_');
	};

	$scope.$on("$destroy", function(){
		$scope.field.value = oldFieldValue;
	});
}