var DEFAULT_VALUES_DATASOURCE = {
	data: [],
	itemsPerPage: 500,
	localKey: ["id"],
	localStorage: true,
	retryCount: 0,
	disableLoader: false
};
var DataSourceVo = function() {
	(function(dataSource) {
		Util.extend(dataSource, Util.clone(DEFAULT_VALUES_DATASOURCE));
	})(this);
};
DataSourceVo.factory = function(name, obj, realName) {
	if (!obj) obj = {};
	if (!obj[name]) {
		obj[name] = {};
	}
	obj[name] = Util.extend(new DataSourceVo(), obj[realName] || obj[name]);
	return obj[name];
};
var DataSource = function(storageStrategy, templateManager, eventEngine, dataSourceHolder, ZHPromise) {

	var self = this;
	var currentPage = 1;
	var _checkedRowsCache = {};

	Object.defineProperty(this, 'checkedRowsCache', {
		set: function(rows) {
			_checkedRowsCache = rows;
		},
		get: function() {
			return _checkedRowsCache;
		}
	});
	var _checkedRows = [];

	this.getVO = function() {
		var VO;
		if (this.name) {
			VO = dataSourceHolder.getDataSourceLazy(this.name);
		}
		return VO ? VO[dataSourceHolder.clearName(this.name)] : VO;
	};

	var getHashKey = function(row) {
		var VO = self.getVO();
		var hash = "";
		if (VO && VO.localKey) {
			hash = VO.localKey.reduce(function(hash, key) {
				return row[key] ? hash + "_" + row[key].toString() : hash;
			}, "");
		}
		return hash;
	};

	Object.defineProperty(this, 'checkedRows', {
		set: function(rows) {
			rows = Util.parseToArray(rows);
			_checkedRows = rows;
			if (rows.length) {
				self.addCheckedRows(rows);
			}
		},
		get: function() {
			return _checkedRows;
		}
	});

	var ignoreKeys = ['__rightActions', '__leftActions'];

	this.addCheckedRows = function(rows) {
		rows = Util.parseToArray(rows);
		rows.forEach(function(row) {
			if (_checkedRows.indexOf(row) === -1) {
				_checkedRows.push(row);
			}
			row.__isSelected = true;
			var hash = getHashKey(row);
			if (hash) {
				_checkedRowsCache[hash] = Util.clone(row, ignoreKeys);
			}
		});
	};

	this.removeCheckedRows = function(rows) {
		rows = Util.parseToArray(rows);
		rows.forEach(function(row) {
			var index = _checkedRows.indexOf(row);
			row.__isSelected = false;
			if (index !== -1) {
				_checkedRows.splice(index, 1);
			}

			var hash = getHashKey(row);
			if (hash) {
				delete _checkedRowsCache[hash];
			}
		});
	};

	this.getCheckedRows = function() {
		var checkedRows = _checkedRows;
		if (this.name) {
			var checkedRowsHash = checkedRows.map(function(row) {
				return getHashKey(row);
			});
			checkedRows = checkedRows.concat(Object.keys(_checkedRowsCache).filter(function(row) {
				return checkedRowsHash.indexOf(row) === -1;
			}).map(function(hash) {
				return _checkedRowsCache[hash];
			}));
		}
		return checkedRows;
	};

	this.clearCheckedRows = function(rows) {
		if (rows) {
			this.removeCheckedRows(rows);
		} else {
			var checkedRows = this.checkedRows;
			if (Util.isArray(checkedRows) && checkedRows.length) {
				checkedRows.forEach(function(row) {
					row.__isSelected = false;
				});
			}
			this.checkedRows = [];
			_checkedRowsCache = {};
		}
	};

	eventEngine.createEvent(this, 'afterLoadDataSource');
	eventEngine.createEvent(this, 'afterLoadDataSourceError');

	this.getCurrentPage = function() {
		return currentPage;
	};
	this.setCurrentPage = function(page) {
		currentPage = page;
	};
	this.getNextPage = function() {
		return ++currentPage;
	};
	this.getPrevPage = function() {
		return --currentPage;
	};
	this.getFirstRow = function() {
		return this.data[0] ? this.data[0] : undefined;
	};
	this.getLastRow = function() {
		var lastRow;
		if (this.data) {
			var lastIndex = this.getLastIndex();
			lastRow = this.data[lastIndex] ? this.data[lastIndex] : undefined;
		}

		return lastRow;
	};
	this.getLastIndex = function getLastIndex() {
		return this.data.length - 1;
	};
	this.newRow = function() {
		if (!this.data) {
			this.data = [];
		}
		var newRow = {
			"__createdLocal": true,
			"__is_new": true
		};
		return this.data.push(newRow);
	};
	this.setData = function setData(data) {
		if (!self.data) {
			self.data = [];
		}
		if (currentPage === 1) {
			self.data = data;
		} else {
			self.data = self.data.concat(data);
		}
		if (self.afterLoadDataSource) {
			self.afterLoadDataSource();
		}
	};
	var _loadUpdateTemplateAfterSuccess = function(data) {
		self.loadError = false;
		templateManager.updateTemplate();
		return data;
	};
	var _loadBuildCallback = function(callback) {
		return function(dataSets) {
			var data;
			var keys = Object.keys(dataSets);

			if (keys && keys.length > 0 && keys[0] !== '0') {
				var dataSetName = self.name.replace('/', '');
				if (dataSets[dataSetName]) {
					data = dataSets[dataSetName];
				}
			} else {
				data = dataSets;
			}

			self.setData(data);
			if (callback) {
				callback(data);
			}
		};
	};
	this.load = function(filter, page, itemsPerPage, callback, errorCallback, retryCount, timeout, retryCallback) {
		if (!filter) {
			filter = [];
		}
		page = page || currentPage;
		itemsPerPage = itemsPerPage || self.itemsPerPage;

		if (page === DataSource.LOAD_ALL) {
			page = null;
			itemsPerPage = null;
		} else if (page <= 0) {
			page = 1;
		}

		retryCount = retryCount || self.retryCount;
		timeout = timeout || self.timeout;
		var loadPromise;
		if (storageStrategy) {
			callback = _loadBuildCallback(callback);
			loadPromise = storageStrategy.getDataSource(this.name, callback, filter, page, itemsPerPage, errorCallback, retryCount, timeout, retryCallback, this.disableLoader)
				.then(_loadUpdateTemplateAfterSuccess, fireAfterLoadDataSourceError);
		} else {
			loadPromise = ZHPromise.when();
		}

		return loadPromise;
	};

	function fireAfterLoadDataSourceError(err) {
		self.loadError = true;
		self.data = [];

		if (Util.isFunction(self.afterLoadDataSourceError)) {
			self.afterLoadDataSourceError({
				data: err
			});
		}
	}

	this.storage = function(callback, retryCount, timeout, retryCallback) {
		retryCount = retryCount || self.retryCount;
		timeout = timeout || self.timeout;
		if (storageStrategy) {
			return storageStrategy.setDataSource(self.name, self.data, callback, retryCount, timeout, retryCallback);
		} else {
			return ZHPromise.when();
		}
	};

	this.reload = function() {
		return this.load();
	};

	this.getStorageStrategy = function() {
		return storageStrategy;
	};

	this.filter = function(filter, callback, errorCallback, retryCount, timeout, retryCallback) {
		if (!Util.isFunction(callback)) {
			callback = function() {};
		}
		retryCount = retryCount || self.retryCount;
		timeout = timeout || self.timeout;
		var filterPromise;

		if (storageStrategy) {
			filterPromise = storageStrategy.getDataSource(self.name, callback, filter, currentPage, this.itemsPerPage, errorCallback, retryCount, timeout, retryCallback)
				.then(onDataSourceFilterSuccess);
		} else {
			filterPromise = ZHPromise.when();
		}

		return filterPromise;
	};

	function onDataSourceFilterSuccess(data) {
		if (data.dataset) {
			var cleanName = dataSourceHolder.clearName(self.name);
			var dataset = data.dataset[cleanName];
			if (!dataset) {
				dataset = data.dataset[Util.getPackageUrl(cleanName)];
				if (!dataset) {
					dataset = data.dataset[dataSourceHolder.realName(cleanName)] || [];
				}
			}
			return dataset;
		}
		return data;
	}

	this.order = function(fields) {
		if (fields) {
			fields.forEach(function() {});
		}
	};
	this.save = function() {
		return this.storage();
	};
	this.remove = function(row) {
		if (storageStrategy) {
			return storageStrategy.remove(self.name, row, function() {}).then(function(result) {
				self.removeCheckedRows(row);
				return result;
			});
		} else {
			return ZHPromise.when();
		}
	};
	this.sync = function() {
		if (storageStrategy) {
			return storageStrategy.sync(self.name, function() {});
		} else {
			return ZHPromise.when();
		}
	};
	this.unSelectAll = function(rows) {
		if (rows) {
			rows.forEach(function(row) {
				row.__isSelected = false;
			});
		} else {
			this.data.forEach(function(row) {
				row.__isSelected = false;
			});
		}
		this.clearCheckedRows(rows);
		this.updateCheckedRows();
		return this;
	};
	this.selectAll = function(rows) {
		if (rows) {
			rows.forEach(function(row) {
				if (!row.__notSelectable) {
					self.addCheckedRows(row);
				}
			});
		} else {
			this.data.forEach(function(row) {
				if (!row.__notSelectable) {
					self.addCheckedRows(row);
				}
			});
		}
		this.updateCheckedRows();
		return this;
	};
	this.updateCheckedRows = function() {
		if (this.data) {
			this.checkedRows = this.data.filter(function(row) {
				row.__isSelected = row.__isSelected || _checkedRowsCache.hasOwnProperty(getHashKey(row));
				return row.__isSelected;
			});
		} else {
			this.clearCheckedRows();
		}
		if (templateManager.container) {
			templateManager.container.updateCheckedRows();
		}
	};
	var _data;

	Object.defineProperty(this, 'data', {
		configurable: true,
		enumerable: true,
		get: function() {
			return _data;
		},
		set: function(val) {
			_data = val;
			this.updateCheckedRows();
		}
	});

	var _setDefaultValuesField = (function(widget) {
		Util.extend(widget, Util.clone(DEFAULT_VALUES_DATASOURCE));
	})(this);

};

DataSource.LOAD_ALL = 'load_all';