var ZeedhiDiretiveDate = angular.module("ZeedhiDiretiveDate", []);

(function() {

	function daysInMonth(iMonth, iYear) {
		return 32 - new Date(iYear, iMonth, 32).getDate();
	}

	function DateCtrl($scope, element, attrs, controller) {
		var dayField = $('.selectDay', element);
		var monthField = $('.selectMonth', element);
		var yearField = $('.selectYear', element);

		this.init = function() {
			this.getDateValue();

			this.loadYears(element);

			dayField.change(this.apply.bind(this));

			monthField.change(this.apply.bind(this));

			yearField.change(this.apply.bind(this));
		};

		this.getDateValue = function() {
			var day = dayField.val();
			var month = monthField.val();
			var year = yearField.val();

			var maxDayForMonth = daysInMonth(month - 1, year);

			$scope.__maxDay__ = maxDayForMonth;

			day = day > maxDayForMonth ? maxDayForMonth : day;

			return day + "/" + month + "/" + year;
		};

		this.loadYears = function() {
			var initialized = element.children().attr("data-future");
			if (!initialized) {
				var newestYearElement = angular.element("[data-newest]", element);
				var newestYear = newestYearElement.val();
				var thisYear = new Date().getFullYear();
				var options = [];
				while (newestYear < (thisYear + 20)) {
					var option = $("<option></option>");
					option.val(++newestYear);
					option.text(newestYear);
					option.data("future");
					options.unshift(option);
				}
				newestYearElement.parent().prepend(options);
			}
		};

		this.apply = function() {
			var value = this.getDateValue();
			var _apply = function() {
				controller.$setViewValue(value);
			};
			if (!($scope.$$phase == '$apply' || $scope.$$phase == '$digest') && !$scope.applyInProgressZh) {
				$scope.$apply(_apply);
			} else {
				_apply();
			}
			element.change();
		};

		this.change = function(value) {
			if (value) {
				var dateValues = value.split('/');
				dayField.val(dateValues[0]);
				monthField.val(dateValues[1]);
				yearField.val(dateValues[2]);
			}
			return value;
		};

	}

	ZeedhiDiretiveDate.directive('zhDateField', function() {
		return {
			require: '?ngModel',
			link: function($scope, element, attrs, controller) {

				var dateCtrl = new DateCtrl($scope, element, attrs, controller);

				dateCtrl.init();

				controller.$parsers.unshift(dateCtrl.change.bind(dateCtrl));

				controller.$formatters.unshift(dateCtrl.change.bind(dateCtrl));

			}
		};
	});
	ZeedhiDiretiveDate.directive("zhDateTime", function(ZhUtil, ScreenService) {
		/*
		 *   Date Time diective for wheel selector. Defaults to a date + time scroller.
		 *   Accepts the parameters described in opt. Consult docs to see what values they accept.
		 */
		return {
			restrict: 'A',
			link: function(scope, element, attrs, ctrl) {
				var field = scope.field || scope.question;
				// @todo parse pt_br to pt-BR, because moment and datetime use i18n with differents formats..
				var lang = ZhUtil.formatLangToMobiscroll(scope.lang);

				function beforeShowPicker(scrollerInstance) {
					change();
					scrollerInstance.setValue(element.val());
				}

				function change() {
					var maxDate = ZhUtil.evaluate(field['datetime-maxDate'] || '2100-01-01 23:59');
					var minDate = ZhUtil.evaluate(field['datetime-minDate'] || '1900-01-01 00:00');
					var defaultDate = ZhUtil.evaluate(field['datetime-defaultDate'] || "today");
					lang = field['datetime-lang'] || lang;

					defaultDate = defaultDate === "today" ? moment() : moment(defaultDate);
					maxDate = maxDate === "today" ? moment() : moment(new Date(maxDate));
					minDate = minDate === "today" ? moment() : moment(new Date(minDate));

					var opt = {
						preset: field['datetime-preset'] || 'datetime',
						dateOrder: field['datetime-dateOrder'],
						dateFormat: field['datetime-dateFormat'],
						timeWheels: field['datetime-timeWheels'],
						timeFormat: field['datetime-timeFormat'],
						defaultValue: defaultDate._d,
						minDate: minDate._d,
						maxDate: maxDate._d,
						stepMinute: field['datetime-stepMinute'] || 1,
						theme: 'default',
						mode: 'scroller',
						lang: lang ? (lang === 'en-US' ? '' : lang) : 'pt-BR',
						hourText: "Hours",
						minuteText: "Minutes",
						secText: "Seconds",
						display: field['datetime-display'] || 'bottom',
						animate: field['datetime-animate'] || '',
						headerText: ScreenService.i18n(field.label) || '',
						onBeforeShow: beforeShowPicker
					};

					if(!field.isReadOnly()){
						element.mobiscroll(opt);
					}
				}

				element.focus(change);
				angular.element(document).ready(change);
				scope.$watch('field', change);

				scope.clearField = function(field){
					if (!field.isReadOnly()) {
						field.widget.currentRow[field.name] = "";

						if(field.change){
							field.change();
						}
					}
				};
			}
		};
	});

	ZeedhiDiretiveDate.directive("zhBetweenDates", function(ZhUtil, ScreenService, templateManager) {
		return {
			restrict: 'A',
			link: function($scope, element, attrs, ctrl) {
				var field = $scope.field;
				var startField = element.find("input:first");
				var endField = element.find("input:last");
				var errorObj = {
					message: field.invalidIntervalMessage || ScreenService.i18n('The first date must be lower than second.')
				};

				var addErrorMessage = function(field) {
					if (!field.widget.currentRow.$error) {
						field.widget.currentRow.$error = {};
					}
					if (!field.widget.currentRow.$error[field.name]) {
						field.widget.currentRow.$error[field.name] = [];
					}
					var notExists = true;
					for (var i in field.widget.currentRow.$error[field.name]) {
						if (field.widget.currentRow.$error[field.name][i] === errorObj) {
							notExists = false;
							break;
						}
					}
					if (notExists) {
						field.widget.currentRow.$error[field.name].push(errorObj);
					}
				};

				var removeErrorMessage = function(field) {
					if (field.widget.currentRow.$error && field.widget.currentRow.$error[field.name]) {
						for (var i in field.widget.currentRow.$error[field.name]) {
							if (field.widget.currentRow.$error[field.name][i] === errorObj) {
								field.widget.currentRow.$error[field.name].splice(i, 1);
							}
						}
					}
				};

				var change = function(e) {
					if (this.blur) {
						this.blur();
					}

					var rowValue = {
						start: startField.val(),
						end: endField.val()
					};

					if (field.widget.currentRow) {
						field.widget.currentRow[field.name] = rowValue;

						var startDate = rowValue.start.split('/').reverse().join('-');
						var endDate = rowValue.end.split('/').reverse().join('-');

						if (moment(new Date(startDate)).isAfter(new Date(endDate))) {
							addErrorMessage(field);
						} else {
							removeErrorMessage(field);
						}

						if (field.change) {
							field.change();
						}
						templateManager.updateTemplate();
					}
				};

				var changeValueInUi = function(rowValue) {
					if (rowValue && rowValue.start) {
						startField.val(rowValue.start);
					}

					if (rowValue && rowValue.end) {
						endField.val(rowValue.end);
					}
				};

				startField.focus(change);
				endField.focus(change);
				$scope.$watch('field.widget.currentRow.' + field.name, changeValueInUi);
			}
		};
	});


})();