var ZeedhiFields = angular.module("ZeedhiFields");

ZeedhiFields.directive('zhSelectMultiple', function($compile, $timeout, templateManager, Query, ZHPromise, ZhUtil) {
	return {
		link: function($scope, element) {
			// these options must be replaced by dataSource values
			var options = [];
			var field = $scope.field;
			var reloadFunction = field.reload.bind(field);
			field.reload = function reload(addictionalFilters){
				if(field.selectWidget && field.selectWidget.resetInfinityScroll){
					field.selectWidget.resetInfinityScroll();
				}
				return reloadFunction(addictionalFilters);
			};

			var getValueField = function() {
				return Util.isArray(field.valueFields) ? field.valueFields : field.valueField;
			};

			var getDescriptionField = function() {
				return field.descriptionField;
			};

			element.on('keydown', function(e) {
				return e.ctrlKey || e.keyCode == 9 || (e.keyCode >= 112 && e.keyCode <= 126); // TAB key
			});

			element.on('input', function(e) {
				element.text("");
				return false;
			});

			$scope.blurInput = function() {
				element.blur();
			};

			$scope.removeOption = function(evt) {
				var elm = angular.element(evt.currentTarget);
				if (!field.isReadOnly()) {
					var row = options.splice(elm.index(), 1);
					field.dataSource.removeCheckedRows(row);
					$scope.fieldRow.splice(elm.index(), 1);
					elm.animate({
						opacity: 0
					}, 300, function() {
						elm.animate({
							width: 0
						}, 300, buildOptions);
					});
					field.widget.currentRow.changed = true;
					if (field.change) {
						field.change();
					}
				}
				evt.stopPropagation();
			};

			function getRowsToDisplay(element, numberOfRows){
				var rowsCapability = (element.closest('.container').height() - element.offset().top) / 12;
				var rowsToDisplay = (rowsCapability < numberOfRows) ? rowsCapability : numberOfRows;
				return rowsToDisplay;
			}

			function getTooltipText(options, numberOfRows, element){
				var rowsToDisplay = getRowsToDisplay(element, numberOfRows); // number of rows to display in the tooltip
				var more = (rowsToDisplay < numberOfRows) ? "..." : "";
				var firstRowToDisplay = options.length - numberOfRows;
				var lastRowToDisplay = firstRowToDisplay + rowsToDisplay;
				var notVisibleOptions = options.slice(firstRowToDisplay, lastRowToDisplay);
				var content = notVisibleOptions.reduce(function(initValue, option){
					return initValue + option[field.descriptionField] + "<br>";
				}, "");
				return content + more;
			}

			function buildOptions() {
				var container = element.find('.zh-options-container');
				var elementWidth = element.width() - element.find('.zh-icon').outerWidth(); // remove icon space
				var optionsWidth = 0;
				var dField = getDescriptionField();

				// clear options added to element
				container.html('');

				// add options to element until options width be greater than element width
				options.forEach(function(item, index) {
					var option;
					if (field.isReadOnly()) {
						option = $compile('<span class="zh-select-item" contenteditable="false"><span>&times;</span>' + item[dField] + '</span>')($scope);
					} else {
						option = $compile('<span class="zh-select-item" contenteditable="false" zh-touchstart="removeOption(evt)"><span>&times;</span>' + item[dField] + '</span>')(
							$scope);
					}
					container.append(option);
					optionsWidth += option[0].clientWidth + 5;
					if (optionsWidth > elementWidth) {
						return false;
					}
				});

				// adjust the options and add the "more" element
				if (optionsWidth > elementWidth) {
					var more = $compile('<span zh-title-tooltip={{field.tooltipContent}} class="zh-select-item more-items">' + (options.length - container.children().length) + ' mais...</span>')($scope);
					container.append(more);
					optionsWidth += more[0].clientWidth + 5;
					while (optionsWidth > elementWidth && container.children().length > 1) {
						var lastOption = container.children().eq(-2);
						optionsWidth -= lastOption[0].clientWidth + 5;
						lastOption.remove();
						more.html((options.length - container.children().length + 1) + ' mais...');
					}
					field.tooltipContent = getTooltipText(options, options.length - container.children().length + 1, element);
				}
			}

			// bind the window.resize event to rebuild the options if element width changes
			$(window).on("resize.select-multiple", buildOptions);

			function buildLoadUnmatchedRowsInDatasourceFilter(nonMatchedRows) {
				var vField = getValueField();

				var idsByKey = {};
				if (Util.isArray(vField)) {
					vField.forEach(function(key){
						idsByKey[key] = [];
					});
					nonMatchedRows.forEach(function(row) {
						vField.forEach(function(key){
							idsByKey[key].push(row[key]);
						});
					});
				} else {
					idsByKey[vField] = nonMatchedRows;
				}

				var filters = ZhUtil.buildFiltersInField(field);
				var query = Query.build();

				Object.keys(idsByKey).forEach(function(key){
					query.where(key).in(idsByKey[key]);
				});

				return filters.concat(query.where());
			}

			function alreadyHasTheRowInCache(row) {
				return field.dataSource.getCheckedRows().some(function(cachedRow) {
					return Util.parseToArray(getValueField()).every(function(key) {
						return cachedRow[key] == row[key];
					});
				});
			}

			function alreadyHasTheRowInTheOptions(row) {
				return options.some(function(option) {
					return Util.parseToArray(getValueField()).every(function(key) {
						return option[key] == row[key];
					});
				});
			}

			function addOption(row) {
				if(!alreadyHasTheRowInTheOptions(row)) {
					options.push(row);
					field.dataSource.addCheckedRows(row);
				}
			}

			function loadUnmatchedRowsInDatasource(nonMatchedRows) {

				var filters = buildLoadUnmatchedRowsInDatasourceFilter(nonMatchedRows);

				return field.dataSource.filter(filters).then(function(data) {
					if (Array.isArray(data) && data.length) {
						data.forEach(addOption);
					}
				});
			}

			// built options property checking the values (selectedRows) against the datasource
			function buildOptionsFromDataSource(selectedRows) {
				var defer = ZHPromise.defer();

				var vField = getValueField();
				var previousCheckedRows = field.dataSource.getCheckedRows();
				field.dataSource.clearCheckedRows();
				options = [];

				if (field.dataSource.data && field.dataSource.data.length) {
					var unmatchedRows = [];
					if (Util.isArray(selectedRows) && selectedRows.length) {
						selectedRows.forEach(function(valueItem) {
							var row = findRowFromDataSourceOrCheckedRows(field.dataSource.data, previousCheckedRows, vField, valueItem);

							if (row) {
								options.push(row);
								if (!alreadyHasTheRowInCache(row)) {
									field.dataSource.addCheckedRows(row);
								}
							} else {
								unmatchedRows.push(valueItem);
							}
						});

						if (unmatchedRows.length) {
							loadUnmatchedRowsInDatasource(unmatchedRows).then(defer.resolve, defer.resolve);
						} else {
							defer.resolve();
						}
					} else {
						field.dataSource.clearCheckedRows();
						defer.resolve();
					}
				} else {
					options = previousCheckedRows;
					defer.resolve();
				}
				return defer.promise;
			}
			field.buildOptionsFromDataSource = buildOptionsFromDataSource;

			function findRowFromDataSourceOrCheckedRows(dataSourceData, checkedRows, valueField, valueItem) {
				return findInItems(valueField, valueItem, dataSourceData) || findInItems(valueField, valueItem, checkedRows);
			}

			function findInItems(valueField, valueItem, items) {
				return items.filter(function(item) {
					if (Util.isArray(valueField)) {
						return valueField.every(function(key) {
							return item[key] == valueItem[key];
						});
					}
					return item[valueField] == valueItem;
				}).shift();
			}


			var dataSourceWatch = $scope.$watch(function() {

				return field.dataSource.data;

			}, function() {
				if (field.dataSource.data && field.dataSource.data.length) {
					dataSourceWatch();
					buildOptionsFromDataSource((field.value() || [])).then(function() {
						$timeout(function() {
							buildOptions();
						}, 300);
					});
				}
			});

			$scope.$watch(function() {
				return field.value();
			}, function(value) {
				buildOptionsFromDataSource(value).then(function() {
					$timeout(function() {
						buildOptions();
					}, 300);
				});
			});
		}
	};
});